package graphoutput;

import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;

/**
 * Creates plane as a half-circle with normal vector (x, 0, z)
 *
 * @author Peter Steiger
 * @version 1.0
 */
public class CirclePlane extends BranchGroup {
   private Appearance app;
   private Shape3D shape;
   private TransformGroup tg;
   private Color3f color;
   private float transparency;


   /**
    * Creates a half-circle within this BranchGroup with normal
    * vector (x, 0, z) with the specified parameters
    */
   public CirclePlane(double radius, int vertexcnt)
   {
      int[] q = { vertexcnt };
      TriangleFanArray tfa = new TriangleFanArray(vertexcnt, GeometryArray.COORDINATES | GeometryArray.NORMALS, q);

      for (int i=0; i<vertexcnt; i++)
      {
        Point3d p1 = new Point3d();
        double alpha = Math.PI*(double)i/(double)(vertexcnt-1);
        p1.x = radius*Math.cos(alpha);
        p1.y = radius*Math.sin(alpha);
        p1.z = 0.0;

        tfa.setCoordinate(i, p1);
      }

    Appearance app = createAppearance(new Color3f(0.4f, 0.4f, 0.4f), 0.3f);
    shape = new Shape3D(tfa, app);
    shape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);

    tg = new TransformGroup();
    tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    tg.addChild(shape);

    addChild(tg);
   }

  /**
   * Creates an instance of Appearance with the specified
   * attributes.
   */
  private Appearance createAppearance(Color3f color, float transparency)
  {
    this.color=color;
    Appearance app = new Appearance();

    Material mat = new Material(color,
                                new Color3f(0.0f,0.0f,0.0f),
                                color,
                                new Color3f(1.0f,1.0f,1.0f),
                                80.0f);
    mat.setCapability(Material.ALLOW_COMPONENT_READ);
    app.setMaterial(mat);
    app.setCapability(Appearance.ALLOW_MATERIAL_READ);
    app.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
    PolygonAttributes pa = new PolygonAttributes();

    TransparencyAttributes ta = new TransparencyAttributes();
    ta.setTransparencyMode(ta.BLENDED);
    ta.setTransparency(transparency);
    ta.setCapability(TransparencyAttributes.ALLOW_VALUE_READ);
    app.setTransparencyAttributes(ta);

    pa.setCullFace(pa.CULL_NONE);
    app.setPolygonAttributes(pa);

    return app;
  }

  /**
   * Changes the current position and orientation of the plane.
   */
  public void setPosition(Point3d p, Vector3d v)
  {
    v.normalize();

    Transform3D t3d = new Transform3D();

    t3d.rotY(Math.atan2(v.x, v.z));
    t3d.setTranslation(new Vector3d(p.x, p.y, p.z));

    tg.setTransform(t3d);
  }

  /**
   *  Set the color and the transparency of the CirclePlane.
   */
  public void setColor(Color3f col, float transparency)
  {
    this.color = col;
    this.transparency = transparency;
    app = createAppearance(col, transparency);
    shape.setAppearance( app );
  }

  /**
   *  Returns the current color of the CirclePlane.
   */
  public Color3f getColor()
  {
    return color;
  }

  /**
   *  Returns the current transparency of the CirclePlane.
   */
  public float getTransparency()
  {
    return transparency;
  }


}